'use strict';

var react = require('react');
var ResizeObserver = require('resize-observer-polyfill');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var ResizeObserver__default = /*#__PURE__*/_interopDefaultLegacy(ResizeObserver);

// https://github.com/que-etc/resize-observer-polyfill/issues/80
// 由于 ts 官方增加了 contentRect 的类型定义，导致和 resize-observer-polyfill 内部的定义冲突，目前先以 "skipLibCheck" 解决
var DEFAULT_DEP = [];
/**
 * 计算容器内的元素最大展示数量
 * @returns {[number, number, boolean]} [当前测试的展示数, 最终确定的展示数, 是否在测试过程中]
 */
var useOverflow = function (_a,
/** 其它可能会导致需要重新计算展示数的依赖项 */
deps) {
    var containerRef = _a.containerRef, _b = _a.defaultCount, defaultCount = _b === void 0 ? 0 : _b, maxCount = _a.maxCount, _c = _a.minCount, minCount = _c === void 0 ? 0 : _c;
    if (deps === void 0) { deps = DEFAULT_DEP; }
    // the count of trying to show
    var _d = react.useState(function () { return Math.min(defaultCount, maxCount); }), count = _d[0], setCount = _d[1];
    // the count valid to show
    var _e = react.useState(count), measureCount = _e[0], setMeasureCount = _e[1];
    // latest valid count
    var _f = react.useState(null), latestValidCount = _f[0], setLatestValidCount = _f[1];
    // the container element
    var _g = react.useState(true), measuring = _g[0], setMeasuring = _g[1];
    // start to measure
    var startMeasuring = react.useCallback(function () {
        setMeasuring(true);
        setLatestValidCount(null);
    }, []);
    // stop measuring
    var endMeasuring = react.useCallback(function (measureCount) {
        setMeasuring(false);
        setMeasureCount(measureCount);
    }, []);
    react.useEffect(function () {
        // console.log('reset when deps changed');
        startMeasuring();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, deps);
    react.useEffect(function () {
        // reset when latestValidCount changed
        if (latestValidCount === null)
            return;
        if (latestValidCount > maxCount || latestValidCount < minCount)
            startMeasuring();
    }, [minCount, maxCount, latestValidCount, startMeasuring]);
    // save to ref for use in resize observer to avoid frequently create new observer
    var measuringRef = react.useRef(measuring);
    react.useEffect(function () {
        measuringRef.current = measuring;
    }, [measuring]);
    react.useLayoutEffect(function () {
        var containerDOM = containerRef === null || containerRef === void 0 ? void 0 : containerRef.current;
        var resizeObserver = null;
        if (containerDOM) {
            resizeObserver = new ResizeObserver__default["default"](function () {
                // TODO why did this called on initial render?
                if (!measuringRef.current) {
                    startMeasuring();
                    // console.log('reset when width changed');
                }
                //  else {
                //     console.log('lock when width changed');
                // }
            });
            resizeObserver.observe(containerDOM);
        }
        return function () {
            if (resizeObserver) {
                resizeObserver.disconnect();
            }
        };
    }, [containerRef, startMeasuring]);
    // TODO why called twice?
    // console.log({ count, latestValidCount, maxCount, minCount, measuring });
    react.useLayoutEffect(function () {
        var containerDOM = containerRef === null || containerRef === void 0 ? void 0 : containerRef.current;
        if (!containerDOM)
            return;
        // measure end
        if (count === latestValidCount) {
            endMeasuring(latestValidCount);
            return;
        }
        var newCount = count;
        if (containerDOM.offsetWidth >= containerDOM.scrollWidth) {
            setLatestValidCount(count);
            // try to show more
            newCount = Math.min(count + 1, maxCount);
        }
        else if (containerDOM.offsetWidth < containerDOM.scrollWidth) {
            // if container with not enough space event when count is 0, it will be set to 0
            if (count === 0)
                setLatestValidCount(count);
            // try to show less
            newCount = Math.max(0, minCount, count - 1);
        }
        setCount(newCount);
    }, [containerRef, count, endMeasuring, latestValidCount, maxCount, minCount]);
    return [count, measureCount, measuring];
};

module.exports = useOverflow;
//# sourceMappingURL=index.cjs.map
